import { chunkString } from './utils';
const maxSizeMessage = 4096;
export var ChatActions;
(function (ChatActions) {
    ChatActions["Message"] = "message";
    ChatActions["UserInvite"] = "user_invite";
    ChatActions["Invite"] = "invite_conversation";
    ChatActions["Joined"] = "join_conversation";
    ChatActions["Close"] = "close_conversation";
    ChatActions["Leave"] = "leave_conversation";
    ChatActions["Decline"] = "decline_invite";
    ChatActions["Update"] = "update_channel";
    ChatActions["Destroy"] = "destroy";
})(ChatActions || (ChatActions = {}));
export var ConversationState;
(function (ConversationState) {
    ConversationState["Init"] = "init";
    ConversationState["Invite"] = "invite";
    ConversationState["Active"] = "active";
    ConversationState["Pending"] = "pending";
    ConversationState["Closed"] = "closed";
})(ConversationState || (ConversationState = {}));
export class Conversation {
    constructor(client, conversationId, title, members, messages, variables) {
        this.client = client;
        this.conversationId = conversationId;
        this.title = title;
        this.channelId = null;
        this.createdAt = Date.now();
        this.answeredAt = 0;
        this.updatedAt = 0;
        this.invitedAt = 0;
        this.closedAt = 0;
        this.task = null;
        this.data = null;
        this.members = (members || []).map((i) => wrapChannelMember(i));
        this._messages = messages || [];
        this.state = ConversationState.Invite;
        this.variables = variables;
        if (variables &&
            variables.hasOwnProperty('cc_attempt_id') &&
            this.client.agent) {
            this.task = this.client.agent.task.get(+variables.cc_attempt_id) || null;
        }
    }
    setInvite(inviteId, timestamp) {
        this.inviteId = inviteId;
        this.invitedAt = timestamp;
    }
    setAnswered(channelId, timestamp, member) {
        this.state = ConversationState.Active;
        this.answeredAt = timestamp;
        this.channelId = channelId;
        this.member = wrapChannelMember(member);
        this.inviteId = null; // deleted in DB
    }
    setClosed(timestamp) {
        this.state = ConversationState.Closed;
        this.closedAt = timestamp;
    }
    get id() {
        return this.conversationId;
    }
    get messages() {
        return this.getMessages();
    }
    getMessages() {
        return this._messages.map((i) => {
            const msg = {
                id: i.id,
                type: i.type,
                member: this.messageMember(i),
                channelId: i.channel_id,
                createdAt: i.created_at,
                updatedAt: i.updated_at,
            };
            if (i.hasOwnProperty('file')) {
                i.file.url = this.client.fileUrlDownload(i.file.id);
                msg.file = i.file;
            }
            if (i.hasOwnProperty('text')) {
                msg.text = i.text;
            }
            return msg;
        });
    }
    async next() {
        throw new Error('TODO');
    }
    async pagination(page, perPage) {
        throw new Error('TODO');
    }
    newMessage(e) {
        this._messages.push(e);
    }
    get allowDecline() {
        return !!this.inviteId;
    }
    get allowJoin() {
        return !!this.inviteId;
    }
    get allowLeave() {
        return !!this.channelId && this.closedAt === 0;
    }
    get allowReporting() {
        return this.answeredAt > 0 && this.hasReporting;
    }
    get hasReporting() {
        return this.variables && this.variables.cc_reporting === 'true';
    }
    /*
    Actions
     */
    async decline() {
        if (!this.inviteId)
            throw new Error('This conversation is joined');
        return this.client.request(`decline_chat`, {
            invite_id: this.inviteId,
        });
    }
    async join() {
        if (!this.inviteId)
            throw new Error('This conversation is joined');
        return this.client.request(`join_chat`, {
            invite_id: this.inviteId,
        });
    }
    async close(cause) {
        if (!this.channelId)
            throw new Error('This conversation not active');
        return this.client.request(`close_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            cause,
        });
    }
    async leave(cause) {
        if (!this.channelId)
            throw new Error('This conversation not active');
        return this.client.request(`leave_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            cause,
        });
    }
    async sendText(text) {
        if (!this.channelId) {
            throw new Error('conversation not active');
        }
        if (!text.length) {
            throw new Error('empty message');
        }
        const chunk = chunkString(text, maxSizeMessage);
        for (let i = 0; i < chunk.length; i++) {
            if (i + 1 >= chunk.length) {
                return this.sendMessageTextChunk(chunk[i]);
            }
            await this.sendMessageTextChunk(chunk[i]);
        }
    }
    async sendFile(file, cb) {
        const storedFiles = await this.client.storeFile(this.id, [file], cb);
        const f = storedFiles[0];
        // todo bug if chat response error
        return this.client.request(`send_file_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            id: f.id,
            name: file.name,
            mime: f.mime,
            url: f.shared,
            size: f.size,
        });
    }
    async send(data, cb) {
        if (typeof data === 'string') {
            return this.sendText(data);
        }
        else if (data instanceof File) {
            return this.sendFile(data, cb);
        }
        else {
            throw new Error('unknown send data');
        }
    }
    async addToChat(userId, title) {
        return this.client.request(`add_to_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            user_id: userId,
            title,
        });
    }
    async startChat(userId) {
        return this.client.request(`start_chat`, {
            user_id: userId,
        });
    }
    async updateChannel() {
        return this.client.request(`update_channel_chat`, {
            channel_id: this.channelId,
        });
    }
    async reporting(reporting) {
        if (!this.task) {
            throw new Error(`this conversation not in queue`);
        }
        return this.task.reporting(reporting);
    }
    async renew(sec) {
        if (!this.task) {
            throw new Error(`this conversation not in queue`);
        }
        return this.task.renew(sec);
    }
    sendMessageTextChunk(text) {
        return this.client.request(`send_text_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            text,
        });
    }
    messageMember(msg) {
        if (this.member && msg.channel_id === this.channelId) {
            return {
                self: true,
            };
        }
        for (const m of this.members) {
            if (m.id === msg.channel_id) {
                return m;
            }
        }
        return null; // TODO
    }
}
function wrapChannelMember(m) {
    // todo
    m.messenger = m.type;
    return m;
}
//# sourceMappingURL=conversation.js.map